#!/usr/bin/python
# coding: utf-8

##
# Project: ScoPy - The italian card game 'scopa'
# Author: Marco Scarpetta <marcoscarpetta02@gmail.com>
# Copyright: 2011 Marco Scarpeta
# License: GPL-3+
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# On Debian GNU/Linux systems, the full text of the GNU General Public License
# can be found in the file /usr/share/common-licenses/GPL-3.
##

import random
from gi.repository import GLib,Gtk,Gdk,WebKit,GtkClutter,Clutter
import gettext
from gettext import gettext as _
import cairo
import sys,os
import urllib2
import sleekxmpp
import pickle
GtkClutter.init([])

APP_VERSION = '0.4'
APP_NAME = 'scopy'
#impostazioni gettext
gettext.bindtextdomain(APP_NAME, '/usr/share/locale')
gettext.textdomain(APP_NAME)
#recupero percorsi dove si trovano le immagini
percorso = sys.path[0][0:-4]
percorso_tap = percorso+'/data/images/tappeti/'
percorso_carte = percorso+'/data/images/carte/'
percorso_gui = percorso+'/data/ui/'
percorso_doc = percorso+'/doc/'

#funzione per spostamenti
def go_to(actor,x,y,time):
	an = actor.animatev(Clutter.AnimationMode.EASE_IN_OUT_QUAD, time,['x','y'],[x,y])
	return an

#caricamento help
lang = os.environ['LANG'][0:2]

def get_path(name):
	if os.path.exists(percorso_doc+lang+'/'+name):
		return 'file://'+percorso_doc+lang+'/'+name
	else:
		return 'file://'+percorso_doc+'it'+'/'+name
help_games={
	_('Classic scopa'):get_path('classica.html'),
	_('Re Bello'):get_path('re_bello.html')
	}

#nomi dei file delle immagini delle carte
immagini = [
	["0.png", "1d.png", "2d.png", "3d.png", "4d.png", "5d.png", "6d.png", "7d.png", "8d.png", "9d.png", "10d.png"],
	["bg.png", "1c.png", "2c.png", "3c.png", "4c.png", "5c.png", "6c.png", "7c.png", "8c.png", "9c.png", "10c.png"],
	["mazzo.png", "1b.png", "2b.png", "3b.png", "4b.png", "5b.png", "6b.png", "7b.png", "8b.png", "9b.png", "10b.png"],
	["0.png", "1s.png", "2s.png", "3s.png", "4s.png", "5s.png", "6s.png", "7s.png", "8s.png", "9s.png", "10s.png"]
]
varianti = [_('Classic scopa'), _('Cirulla'), _('Cucita'), _('Re Bello')]
#tempo in ms da aspettare dopo che una carta viene giocata, si regola con self.velocita
times=[1,3000,2000,1000]
#multiplayer status images
stocks={'unavailable':Gtk.STOCK_NO,
	'available':Gtk.STOCK_YES,
	'playing':Gtk.STOCK_JUMP_TO}

#caricamento tipi di carte
tipi_di_carte = []
for cartella in os.listdir(percorso_carte):
	if cartella[0] != '.':
		tipi_di_carte.append(cartella)
tipi_di_carte.sort()
#caricamento sfondi
sfondi = []
for cartella in os.listdir(percorso_tap):
	if cartella[0] != '.':
		sfondi.append(cartella[0:-4])
sfondi.sort()

class Obj():
	def __init__(self, stage, carta, opzioni, scoperta):
		self.uid = carta.uid
		if scoperta:
			self.object = Clutter.Texture.new_from_file(percorso_carte+opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
			stage.add_actor(self.object)
		else:
			self.object = Clutter.Texture.new_from_file(percorso_carte+opzioni['carte']+'/'+immagini[1][0])
			stage.add_actor(self.object)
class Carte():
	def __init__(self):
		self.carte = []
	def __getitem__(self, uid):
		if type(uid) == int:
			for carta in self.carte:
				if carta.uid == uid:
					return carta.object
		else:
			for carta in self.carte:
				if carta.object == uid:
					return carta.uid
	def __delitem__(self, uid):
		n=0
		while n < len(self.carte):
				if self.carte[n].uid == uid:
					del self.carte[n]
				n=n+1
	def append(self, obj):
		self.carte.append(obj)

#classe che contiene tutti i metodi necessari alla creazione e funzionamento della gui
class main_win():
	def __init__(self):
		#carica le opzioni
		self.default = {
			'nome':_('Player'),
			'variante':_('Classic scopa'),
			'speed':'3',
			'carte':'Napoletane',
			'sfondo':'verde',
			}
		self.load_opzioni()
		#impostazioni gumpy e finestra principale
		self.window = Gtk.Window()
		self.window.connect('delete-event', Gtk.main_quit)
		self.window.connect('delete-event', self.disconnect)
		self.window.set_position(Gtk.WindowPosition.CENTER)
		table = Gtk.Table(3, 3, False)
		self.embed = GtkClutter.Embed.new()
		#self.embed.add_events(Gdk.EventMask.ALL_EVENTS_MASK)
		self.stage = self.embed.get_stage()
		self.embed.connect('button-press-event',self.button_press)
		table.attach(self.embed, 1, 2, 1, 2)
		#immagine sfondo
		self.back_img = Clutter.Texture.new_from_file(percorso_tap+self.opzioni['sfondo']+'.png')
		self.stage.add_actor(self.back_img)
		#icona
		icon = cairo.ImageSurface.create_from_png(percorso+'/data/icons/icona32.png')
		pixbuf = Gdk.pixbuf_get_from_surface(icon,0,0,icon.get_width(),icon.get_height())
		self.window.set_icon(pixbuf)
		#titolo
		self.window.set_title(_('ScoPy'))
		self.status_bar = Gtk.Statusbar()
		self.dichiarazione = [Gtk.Label(), Gtk.Label()]
		self.vbox = Gtk.VBox()
		self.vbox.pack_end(self.dichiarazione[1],True,True,0)
		table.attach(self.status_bar,0,3,3,4,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(self.vbox,0,3,0,1,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(self.dichiarazione[0],0,3,2,3,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.window.add(table)
		#alcune variabili di controllo 
		self.ultima_presa = [0,[]]
		self.time_go = 500
		self.time = times[int(float(self.opzioni['speed']))]
		self.online_players = {}
		self.giocatore_can_play=0
		self.carte = Carte()
		self.scope = []
		self.n_scope = []
		self.client = None
		self.online_mode = False
		self.playing_user = None
		#calcola le posizioni dove andranno spostate le carte
		self.calc_posizioni()
		for n in range(2):
			img = Clutter.Texture.new_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[0][0])
			img.set_position(self.posizioni['scope'][n][0], self.posizioni['scope'][n][1])
			self.stage.add_actor(img)
			self.scope.append(img)
			text = Clutter.Text()
			text.set_position(self.posizioni['scope'][n][0], self.posizioni['scope'][n][1])
			self.stage.add_actor(text)
			self.n_scope.append(text)
		self.n_scope[0].hide()
		self.n_scope[1].hide()
		#creazione varie finestre
		self.crea_win_inizio()
		self.crea_win_pre()
		self.crea_about()
		self.crea_riepilogo()
		self.crea_win_online()
		self.crea_win_multi()
		self.crea_win_help()
		self.crea_menu()
		self.window.show_all()
		self.show_win_inizio()
	#calcola le posizioni dove andranno spostate le carte
	def calc_posizioni(self):
		sur = cairo.ImageSurface.create_from_png(percorso_carte+self.opzioni['carte']+'/'+immagini[0][1])
		w = sur.get_width()
		h = sur.get_height()
		g = 10
		s = 20
		self.posizioni = {
                        'computer':[],
                        'terra':[],
			'giocatore':[],
                        'terra_c':[0,0,0,0,0,0,0,0,0,0],
                        'prese':[[6*g+5*w,4*g+3*h+2*s,0],[6*g+5*w,g,0]],
			'scope':[[7*g+6*w,4*g+3*h+2*s],[7*g+6*w,g]]
                        }
		n=0
		while n<3:
			self.posizioni['computer'].append([(n+1)*g+n*w,g])
			self.posizioni['giocatore'].append([(n+1)*g+n*w,4*g+3*h+2*s])
			n=n+1
		i=0
		while i<2:
			n=0
			while n<5:
				self.posizioni['terra'].append([(n+1)*g+n*w,(i+2)*g+(i+1)*h+s])
				n=n+1
			i=i+1
		self.pos_sce = []
		n=0
		while n<4:
			self.pos_sce.append([])
			i=0
			while i<7:
				self.pos_sce[n].append([(i+1)*5+i*w,(n+1)*g+n*h])
				i=i+1
			n=n+1
		self.width=8*g+7*w
		self.height=5*g+4*h+2*s
		self.stage.set_size(self.width, self.height)
		self.embed.set_size_request(self.width, self.height)
	#chiamata alla pressione del bottone inizia partita
	def inizia_partita(self,widget=None,data=None):
		self.opzioni['nome']=self.name_entry.get_text()
		self.opzioni['variante']=varianti[self.var_combo.get_active()]
		for obj in self.carte.carte:
			obj.object.destroy()
		self.carte = Carte()
		for n in range(2):
			self.n_scope[n].hide()
			self.scope[n].set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[0][0])
			self.posizioni['prese'][n][2] = 0
		self.posizioni['terra_c']=[0,0,0,0,0,0,0,0,0,0]
		self.save()
		self.win_inizio.hide()
		
		if self.online_mode == True:
			self.client.send_message(mto=self.playing_user,
				mbody='scopy:game request'+self.opzioni['variante'],
				mtype='chat')
		else:
			self.crea_partita()
			self.dichiara(self.partita.dai_carte())
			self.sposta_carte()
	#crea il menu
	def crea_menu(self):
		window = self.window
		menu_dict = (
			(_('File'),(
				(_('New game...'),self.show_win_inizio),
				(_('Online game...'),self.show_win_online)
				)),
			(_('Edit'),(
				(_('Settings...'),self.show_win_pre),
				(_('Show last move'),self.vedi_ultima_presa)
				)),
			(_('Help'),(
				(_("Games' instructions"),self.show_win_help),
				(_('About'),self.show_about)
				))
			)
		menu=Gtk.MenuBar()
		for item in menu_dict:
			s_menu = Gtk.MenuItem()
			s_menu.set_label(item[0])
			m_menu = Gtk.Menu()
			s_menu.set_submenu(m_menu)
			menu.append(s_menu)
			n=0
			for voce in item[1]:
				m_voce = Gtk.MenuItem()
				m_voce.set_label(voce[0])
				m_voce.connect('activate', voce[1])
				m_menu.attach(m_voce,0,1,n,n+1)
				n=n+1
		self.vbox.pack_start(menu,True,True,0)
	### CREAZIONE FINESTRE ####
	def crea_win_inizio(self):
		self.win_inizio = Gtk.Window()
		self.win_inizio.set_title(_('New Game...'))
		self.win_inizio.set_transient_for(self.window)
		self.win_inizio.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.win_inizio.set_border_width(10)
		self.win_inizio.connect('delete-event', self.hide)
		table=Gtk.Table(2,3,False)
		table.attach(Gtk.Label(_('Name:')),0,1,0,1,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		table.attach(Gtk.Label(_('Variant:')),0,1,1,2,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		self.name_entry=Gtk.Entry()
		self.var_combo=Gtk.ComboBoxText()
		for variante in varianti:
			self.var_combo.append_text(variante)
		table.attach(self.name_entry,1,2,0,1,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(self.var_combo,1,2,1,2,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		button=Gtk.Button()
		button.set_label(_('Start game!'))
		button.connect('pressed',self.inizia_partita)
		button.connect('activate',self.inizia_partita)
		table.attach(button,1,2,2,3,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.win_inizio.add(table)
		self.name_entry.set_property("activates-default",True)
		button.set_can_default(True)
		button.grab_default()
		onlineb=Gtk.Button()
		onlineb.set_label(_('Online game...'))
		onlineb.connect('pressed',self.show_win_online)
		table.attach(onlineb,0,1,2,3,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
	def crea_win_pre(self):
		self.win_pre = Gtk.Window()
		self.win_pre.set_title(_('Edit Settings'))
		self.win_pre.set_transient_for(self.window)
		self.win_pre.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.win_pre.set_border_width(10)
		self.win_pre.connect('delete-event', self.hide)
		table=Gtk.Table(2,5,False)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		table.attach(Gtk.Label(_('Chose the type of cards:')),0,1,0,1,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		table.attach(Gtk.Label(_('Speed:')),0,1,1,2,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		table.attach(Gtk.Label(_('Background:')),0,1,2,3,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.carte_combo=Gtk.ComboBoxText()
		for tipo in tipi_di_carte:
			self.carte_combo.append_text(tipo)
		self.sfondi_combo=Gtk.ComboBoxText()
		for sfondo in sfondi:
			self.sfondi_combo.append_text(sfondo)
		self.velocita=Gtk.HScale.new_with_range(1,3,1)
		table.attach(self.carte_combo,1,2,0,1,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(self.velocita,1,2,1,2,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(self.sfondi_combo,1,2,2,3,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		button=Gtk.Button()
		button.set_label(_('OK'))
		button.connect('pressed',self.modifica_preferenze)
		table.attach(button,1,2,4,5,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.win_pre.add(table)
	def crea_about(self):
		self.about = Gtk.AboutDialog()
		self.about.set_title(_('About'))
		self.about.set_transient_for(self.window)
		self.about.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.about.set_border_width(10)
		self.about.connect('delete-event', self.hide)
		self.about.connect('close', self.hide)
		self.about.connect('response', self.hide)
		self.about.set_program_name('ScoPy')
		self.about.set_version(APP_VERSION)
		self.about.set_copyright('Copyright © 2011 Marco Scarpetta <marcoscarpetta02@gmail.com>')
		self.about.set_comments(_('The italian card game "scopa"'))
		self.about.set_license_type(Gtk.License.GPL_3_0)
		self.about.set_website('http://scopyfreesoftware.wordpress.com')
		self.about.set_authors(['Marco Scarpetta','Si ringrazia la comunità ubuntu-it e archlinux italia'])
		self.about.set_artists(['Marco Scarpetta',
			'crx (carte Piacentine)',
			'Scio Nescire (carte da Poker)',
			"00 ubuntu (carte 'Francitalia' e 'Scartini')",
			'vaillant (carte Siciliane)',
			'gikbuntu (carte Bergamasche)',
			'mapreri (carte Trevisane)',
			'Magog (sfondi, carte Toscane)'])
		self.about.set_translator_credits('Fitoschido \nSarahSlean \nMarti Bosch')
		image = cairo.ImageSurface.create_from_png(percorso+'/data/icons/icona.png')
		icona = Gdk.pixbuf_get_from_surface(image,0,0,100,100)
		self.about.set_logo(icona)
	def crea_riepilogo(self):
		self.rie = Gtk.Dialog()
		self.rie.set_title(_('Game summary'))
		self.rie.set_transient_for(self.window)
		self.rie.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.rie.set_border_width(10)
		self.rie.add_button(_('OK'), 0)
		self.rie.set_default_response(0)
		self.rie.connect('response', self.end_riepilogo)
		hbox = self.rie.get_content_area()
		box=Gtk.HBox()
		hbox.pack_start(box,True,True,5)
		box.set_spacing(5)
		self.rie_labels=[Gtk.Label(),Gtk.Label(),Gtk.Label()]
		box.pack_start(self.rie_labels[0],True,True,5)
		box.pack_start(self.rie_labels[1],True,True,5)
		box.pack_start(self.rie_labels[2],True,True,5)
	def crea_win_online(self):
		self.win_online = Gtk.Window()
		self.win_online.set_title(_('Online game...'))
		self.win_online.set_transient_for(self.window)
		self.win_online.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.win_online.connect('delete-event', self.hide)
		self.table_log=Gtk.Table(2,8,False)
		self.online_info = Gtk.Label()
		lab_usr=Gtk.Label(_('Username:'))
		lab_pwd=Gtk.Label(_('Password:'))
		lab_pwd_re=Gtk.Label(_('Repeat password:'))
		lab_re=Gtk.Label(_('Remember account:'))
		self.table_log.attach(self.online_info,0,2,7,8,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(lab_usr,0,1,1,2,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(lab_pwd,0,1,2,3,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(lab_pwd_re,0,1,3,4,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(lab_re,0,1,4,5,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.username=Gtk.Entry()
		self.password=Gtk.Entry()
		self.password.set_visibility(False)
		self.re_password=Gtk.Entry()
		self.re_password.set_visibility(False)
		self.remember=Gtk.CheckButton()
		self.table_log.attach(self.username,1,2,1,2,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		self.table_log.attach(self.password,1,2,2,3,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		self.table_log.attach(self.re_password,1,2,3,4,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		self.table_log.attach(self.remember,1,2,4,5,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		login=Gtk.Button.new_with_label(_('Login'))
		login.connect('pressed',self.show_login)
		reg=Gtk.Button.new_with_label(_('Register'))
		reg.connect('pressed',self.show_register)
		button=Gtk.Button.new_with_label(_('OK'))
		button.connect('pressed',self.logged)
		button.connect('activate',self.logged)
		button_re=Gtk.Button.new_with_label(_('OK'))
		button_re.connect('pressed',self.registred)
		self.password.set_property("activates-default",True)
		self.username.set_property("activates-default",True)
		self.spinner = Gtk.Spinner()
		self.table_log.attach(login,0,1,0,1,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(reg,1,2,0,1,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(button,1,2,5,6,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(button_re,1,2,6,7,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.table_log.attach(self.spinner,0,1,5,6,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.win_login={True:[lab_usr,lab_pwd,self.username,self.password,lab_re,self.remember,button],
			False:[lab_pwd_re,login,reg,self.re_password,button_re,self.online_info,self.spinner]}
		self.win_reg={True:[lab_usr,lab_pwd,lab_pwd_re,self.username,self.password,self.re_password,lab_re,self.remember,button_re],
			False:[login,reg,button,self.spinner,self.online_info]}
		self.win_log_reg={True:[login,reg],
			False:[lab_pwd_re,lab_usr,lab_pwd,self.username,self.password,lab_re,self.remember,button,self.re_password,button_re,self.spinner,self.online_info]}
		self.win_online.add(self.table_log)
		button.set_can_default(True)
		button.grab_default()
	def crea_win_multi(self):
		self.win_multi = Gtk.Window()
		self.win_multi.set_title(_('People'))
		self.win_multi.set_transient_for(self.window)
		self.win_multi.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.win_multi.connect('delete-event', self.hide)
		self.win_multi.resize(400,300)
		table = Gtk.Table(2,2)
		self.players_list = Gtk.ListStore(str,int,int,str)
		tree_view = Gtk.TreeView.new_with_model(self.players_list)
		cell = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn(_('User'), cell, text = 0)
		column.set_sort_column_id(0)
		tree_view.append_column(column)
		cell = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn(_('Won'), cell, text = 1)
		column.set_sort_column_id(1)
		tree_view.append_column(column)
		cell = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn(_('Lost'), cell, text = 2)
		column.set_sort_column_id(2)
		tree_view.append_column(column)
		cell = Gtk.CellRendererPixbuf()
		column = Gtk.TreeViewColumn(_('Status'), cell, stock_id = 3)
		column.set_sort_column_id(3)
		tree_view.append_column(column)
		tree_view.connect("row-activated",self.game_request)
		scrolledwindow = Gtk.ScrolledWindow()
		scrolledwindow.set_hexpand(True)
		scrolledwindow.set_vexpand(True)
		scrolledwindow.add(tree_view)
		table.attach(scrolledwindow,0,2,0,1)
		button=Gtk.Button()
		button.set_label(_('Disconnect'))
		button.connect('pressed',self.disconnect)
		table.attach(button,1,2,1,2,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.win_multi.add(table)
	def crea_win_help(self):
		self.win_help = Gtk.Window()
		self.win_help.set_title(_('Help'))
		self.win_help.set_transient_for(self.window)
		self.win_help.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
		self.win_help.set_border_width(10)
		self.win_help.connect('delete-event', self.hide)
		self.win_help.resize(600,600)
		table=Gtk.Table(2,2,False)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		self.games = Gtk.ListStore(str)
		self.tree_view = Gtk.TreeView.new_with_model(self.games)
		cell = Gtk.CellRendererText()
		column = Gtk.TreeViewColumn(_('Games'), cell, text = 0)
		column.set_sort_column_id(0)
		self.tree_view.append_column(column)
		for game in help_games:
			self.games.append([game])
		selection = self.tree_view.get_selection()
		selection.connect('changed', self.set_help_text)
		button=Gtk.Button()
		button.set_label(_('Done'))
		button.connect('pressed',self.hide_d,self.win_help)
		scrolledwindow = Gtk.ScrolledWindow()
		scrolledwindow.set_hexpand(True)
		scrolledwindow.set_vexpand(True)
		self.text_view = WebKit.WebView()
		self.text_view.connect("document-load-finished", self.cambia_tree_help)
		scrolledwindow.add(self.text_view)
		table.attach(self.tree_view,0,1,0,1,Gtk.AttachOptions(4),Gtk.AttachOptions(4))
		table.attach(scrolledwindow,1,2,0,1)
		table.attach(button,1,2,1,2,Gtk.AttachOptions(0),Gtk.AttachOptions(0))
		self.win_help.add(table)
	### FUNZIONI CHE MOSRTANO LE FINESTRE ALLE RISPETTIVE PRESSIONI DELLE VOCI DEI MENU ####
	def show_win_inizio(self,widget=None):
		self.var_combo.set_active(varianti.index(self.opzioni['variante']))
		self.name_entry.set_text(self.opzioni['nome'])
		self.win_inizio.show_all()
	def show_win_pre(self,widget=None):
		self.carte_combo.set_active(tipi_di_carte.index(self.opzioni['carte']))
		self.sfondi_combo.set_active(sfondi.index(self.opzioni['sfondo']))
		self.velocita.set_value(int(float(self.opzioni['speed'])))
		self.win_pre.show_all()
	def show_about(self, widget=None):
		self.about.show()
	def show_riepilogo(self, colonne=None):
		for obj in self.carte.carte:
			obj.object.destroy()
		self.carte = Carte()
		for n in range(2):
			self.n_scope[n].hide()
			self.scope[n].set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[0][0])
			self.posizioni['prese'][n][2] = 0
		self.posizioni['terra_c']=[0,0,0,0,0,0,0,0,0,0]
		if colonne == None:
			punti = self.partita.conta_punti()
			colonne = ['\n',self.partita.giocatore[0].nome+'\n','Computer'+'\n']
			for voce in punti:
				if voce != 'Parziale':
					colonne[0] = colonne[0] + voce + '\n'
					colonne[1] = colonne[1] + str(punti[voce][0]) + '\n'
					colonne[2] = colonne[2] + str(punti[voce][1]) + '\n'
			colonne[0] = colonne[0] + 'Parziale' + '\n'
			colonne[1] = colonne[1] + str(punti['Parziale'][0]) + '\n'
			colonne[2] = colonne[2] + str(punti['Parziale'][1]) + '\n'
			colonne[0] = colonne[0] + 'Punti'
			colonne[1] = colonne[1] + str(self.partita.giocatore[0].punti)
			colonne[2] = colonne[2] + str(self.partita.giocatore[1].punti)
		self.rie_labels[0].set_text(colonne[0])
		self.rie_labels[1].set_text(colonne[1])
		self.rie_labels[2].set_text(colonne[2])
		if self.partita.ide == 1 and self.online_mode == True:
			self.client.send_message(mto=self.playing_user,
				mbody='scopy:riepilogo'+pickle.dumps(colonne),
				mtype='chat')
		self.rie.show_all()
	def show_win_online(self,wid=None):
		self.win_inizio.hide()
		if self.client != None:
			self.show_win_multi()
		elif 'username' in self.opzioni:
			for widget in self.win_login[True]:
				widget.show()
			for widget in self.win_login[False]:
				widget.hide()
			self.table_log.show()
			self.username.set_text(self.opzioni['username'])
			self.password.set_text(self.opzioni['password'])
			self.win_online.show()
		else:
			for widget in self.win_log_reg[True]:
				widget.show()
			for widget in self.win_log_reg[False]:
				widget.hide()
			self.table_log.show()
			self.win_online.show()
	def show_login(self,wid=None):
		for widget in self.win_login[True]:
			widget.show()
		for widget in self.win_login[False]:
			widget.hide()
		self.win_online.show()
	def show_register(self,wid=None):
		for widget in self.win_reg[True]:
			widget.show()
		for widget in self.win_reg[False]:
			widget.hide()
		self.win_online.show()
	def show_win_multi(self):
		self.win_multi.show_all()
	def show_win_help(self,wid=None):
		self.win_help.show_all()
	def show_win_request(self, req):
		pass
	### FUNZIONI PER IL MULTIPLAYER ####
	def logged(self,wid=None):
		if self.remember.get_active():
			self.opzioni['username']=self.username.get_text()
			self.opzioni['password']=self.password.get_text()
			self.save()
		self.spinner.start()
		self.spinner.show()
		self.client = sleekxmpp.ClientXMPP(self.username.get_text()+'@jabber.rootbash.com',self.password.get_text())
		self.client.register_plugin('xep_0030')
		self.client.register_plugin('xep_0004')
		self.client.register_plugin('xep_0060')
		self.client.register_plugin('xep_0199')
		self.client.add_event_handler('session_start', self.session_start)
		self.client.add_event_handler("changed_status", self.changed_status)
		self.client.add_event_handler("message", self.message)
		if self.client.connect():
			self.client.process(threaded=True)
			print "Done"
		else:
			print "Unable to connect."
	
	def session_start(self, event):
		print 'connected'
		self.client.get_roster()
		self.client.send_presence()
		self.load_online_players()
		contatti = self.client.client_roster.keys()
		for player in self.online_players:
			if player != self.username.get_text()+'@jabber.rootbash.com' and player not in contatti:
				self.client.send_presence(pto=player, ptype='subscribe')
		treeiter = self.players_list.get_iter_first()
		ntree = treeiter
		while ntree:
			ntree=self.players_list.iter_next(treeiter)
			self.players_list.remove(treeiter)
			treeiter=ntree
		for contatto in contatti:
			self.players_list.append([contatto,self.online_players[contatto]['vinte'],self.online_players[contatto]['perse'],stocks[self.online_players[contatto]['stato']]])
		urllib2.urlopen('http://marcoscarpetta.altervista.org/scopy/insert.php?jid='+self.username.get_text()+'@jabber.rootbash.com')
		self.win_online.hide()
		self.spinner.stop()
		self.spinner.hide()
		self.show_win_multi()
	def changed_status(self, presence):
		if presence['type'] not in ('subscribe', 'subscribed', 'unsubscribe', 'unsubscribed'):
			if presence['status'] != '':
				stato = presence['status']
			else:
				stato = presence['type']
			treeiter = self.players_list.get_iter_first()
			while treeiter:
				if self.players_list.get_value(treeiter,0) == presence['from'].bare:
					self.players_list.set_value(treeiter,3,stocks[stato])
					break
				treeiter=self.players_list.iter_next(treeiter)
	def registred(self,wid=None):
		result=urllib2.urlopen('https://jabber.rootbash.com:9091/plugins/registration/sign-up.jsp?username='+
			self.username.get_text()+
			'&name=&email=&password='+
			self.password.get_text()+
			'&passwordConfirm='+
			self.re_password.get_text()+
			'&create=Create+Account')
		result = result.read()
		n=0
		while n < len(result):
			if result[n] == '<':
				if result[n:n+26] == '<div class="jive-success">':
					result = True
					break
			n=n+1
		if result == True:
			self.online_info.set_text(_('Account successfully created!'))
			self.online_info.show()
			self.logged()
		else:
			self.online_info.set_text(_('Registration error'))
			self.online_info.show()
	def game_request(self, widget, path, view_column):
		statuses={Gtk.STOCK_NO:'unavailable',
			Gtk.STOCK_YES:'available',
			Gtk.STOCK_JUMP_TO:'playing'}
		if statuses[self.players_list[path][3]] not in ['playing','unavailable']:
			self.online_mode = True
			self.playing_user = self.players_list[path][0]
			self.client.send_presence(ptype='xa', pstatus='playing')
			self.show_win_inizio()
		else:
			self.show_dialog(_('Online game request'),
				(Gtk.STOCK_OK,0),
				_("You can't play with %(player)s because he is '%(game)s'" %{'player':self.players_list[path][0], 'game':statuses[self.players_list[path][3]]}))
	def show_dialog(self, title,buttons,lab,msg=None):
		dialog = Gtk.Dialog(title,
			self.window,
			Gtk.DialogFlags.MODAL,
			buttons)
		label=Gtk.Label(lab)
		dialog.get_content_area().add(label)
		dialog.connect('response', self.response, msg)
		dialog.show_all()
	def message(self, msg):
		body = msg['body']
		if msg['type'] in ('chat', 'normal'):
			if body[0:6] == 'scopy:':
				body = body[6:]
				if body[0:12] == 'game request':
					self.show_dialog(_('Online game request'),
						(Gtk.STOCK_YES,0,Gtk.STOCK_NO,1),
						_('%(player)s wants to play a "%(game)s" game with you.\nDo you accept the challenge?' %{'player':msg['from'].bare, 'game':body[12:]}),
						msg)
				if body[0:13] == 'game accepted':
					self.show_dialog(_('Online game request'),
						(_('Start game!'),0),
						_('%s accepted your game request.' %msg['from'].bare),
						msg)
				if body[0:] == 'game refused':
					self.online_mode = False
					self.show_dialog(_('Online game request'),
						(Gtk.STOCK_OK,0),
						_('%s refused your game request.' %msg['from'].bare),
						msg)
				if body[0:7] == 'partita':
					data = pickle.loads(body[7:])
					data[0].giocatore[0],data[0].giocatore[1] = data[0].giocatore[1],data[0].giocatore[0]
					data[0].ide = int(not data[0].ide)
					self.partita = data[0]
					#FIXME brutto
					try:
						a=self.carte[self.partita.giocatore[0].mano.carte[0].uid].get_x()
						if len(data)>1:
							self.gioca_computer(data[1])
					except:
						try:
							a=self.carte[self.partita.giocatore[1].mano.carte[0].uid].get_x()
							if len(data)>1:
								self.gioca_computer(data[1])
						except:
							self.sposta_carte()
							if len(data)>1:
								GLib.timeout_add(1000, self.gioca_computer, data[1])
				if body[0:9] == 'riepilogo':
					data = pickle.loads(body[9:])
					self.show_riepilogo(data)
	def response(self, dialog, ide, msg):
		dialog.destroy()
		if msg != None:
			body = msg['body'][6:]
			if body[0:12] == 'game request':
				if ide == 0:
					self.online_mode = True
					self.playing_user = msg['from'].bare
					self.client.send_presence(ptype='xa', pstatus='playing')
					msg.reply('scopy:game accepted'+self.opzioni['nome']).send()
				if ide == 1:
					msg.reply('scopy:game refused').send()
			if body[0:13] == 'game accepted':
				self.crea_partita(body[13:])
				self.dichiara(self.partita.dai_carte())
				self.sposta_carte()
				if self.partita.ide == 1:
					self.client.send_message(mto=self.playing_user,
						mbody='scopy:partita'+pickle.dumps([self.partita]),
						mtype='chat')
	def disconnect(self, wid=None, data=None):
		if self.client != None:
			a=self.client
			a.disconnect()
			self.client=None
			self.win_multi.hide()
			self.online_mode=False
	def load_online_players(self):
		get = urllib2.urlopen('http://marcoscarpetta.altervista.org/scopy/get.php')
		import string
		data = string.split(get.read(),' ')
		n=0
		while n<len(data)/3:
			self.online_players[data[n*3]] = {'vinte':int(data[n*3+1]),'perse':int(data[n*3+2]),'stato':'unavailable'}
			n=n+1
	#modifica le preferenze in base ai valori dati dall'utente
	def modifica_preferenze(self, widget=None):
		self.win_pre.hide()
		if self.opzioni['carte'] != tipi_di_carte[self.carte_combo.get_active()]:
			self.opzioni['carte'] = tipi_di_carte[self.carte_combo.get_active()]
			a=self.posizioni['terra_c']
			self.calc_posizioni()
			self.posizioni['terra_c']=a
			for carta in self.partita.carte_terra.carte:
				self.carte[carta.uid].set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
			n=0
			while n < len(self.posizioni['terra_c']):
				if self.posizioni['terra_c'][n] != 0:
					self.posizioni['terra_c'][n].set_position(self.posizioni['terra'][n][0], self.posizioni['terra'][n][1])
				n=n+1
			for carta in self.partita.giocatore[0].mano.carte:
				self.carte[carta.uid].set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
			for carta in self.partita.giocatore[1].mano.carte:
				self.carte[carta.uid].set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[1][0])
		self.opzioni['speed']=str(self.velocita.get_value())
		self.time = times[int(float(self.opzioni['speed']))]
		self.opzioni['sfondo']=sfondi[self.sfondi_combo.get_active()]
		self.back_img.set_from_file(percorso_tap+self.opzioni['sfondo']+'.png')
		self.save()
		self.window.resize(10,10)
	#nasconde la finestra di riepilogo e procede con il gioco
	def end_riepilogo(self, widget=None, data=None):
		self.hide(self.rie)
		if self.partita.giocatore[0].punti >= self.partita.punti_vit or self.partita.giocatore[1].punti >= self.partita.punti_vit and self.partita.giocatore[0].punti != self.partita.giocatore[1].punti:
			if self.partita.giocatore[0].punti > self.partita.giocatore[1].punti:
				self.comunica(_('You won!'), 2000)
				if self.online_mode == True:
					get = urllib2.urlopen('http://marcoscarpetta.altervista.org/scopy/update.php?jid='+self.username.get_text()+'@jabber.rootbash.com&vinte=1&perse=0')
			else:
				self.comunica(_('I won!'), 2000)
				if self.online_mode == True:
					get = urllib2.urlopen('http://marcoscarpetta.altervista.org/scopy/update.php?jid='+self.username.get_text()+'@jabber.rootbash.com&vinte=0&perse=1')
			GLib.timeout_add(2000, self.show_win_inizio)
		elif self.online_mode == False:
			self.partita.azzera()
			self.dichiara(self.partita.dai_carte())
			self.sposta_carte()
		elif self.online_mode == True and self.partita.ide == 1 and len(self.carte.carte) == 0:
			self.partita.azzera()
			self.dichiara(self.partita.dai_carte())
			self.sposta_carte()
			self.giocatore_can_play = 1
	#carica la pagina dell'help selezionata
	def set_help_text(self, sel):
		treeiter = sel.get_selected()[1]
		self.text_view.load_uri(help_games[self.games.get_value(treeiter,0)])
	#controlla che la pagina caricata sia la stessa selezionata nel menu, in caso contrario la cambia
	def cambia_tree_help(self, view, frame):
		for game in self.games:
			treeiter = self.games.get_iter_first()
			while treeiter:
				if help_games[self.games.get_value(treeiter,0)] == view.get_uri():
					self.tree_view.set_cursor(self.games.get_path(treeiter),None,False)
					break
				treeiter=self.games.iter_next(treeiter)
	#nasconde 'widget'
	def hide(self, widget, data=None):
		widget.hide()
		return True
	#nasconde 'data'
	def hide_d(self, widget, data):
		data.hide()
	#funzioni per il salvataggio e caricamento delle opzioni
	def load_opzioni(self):
		d = os.path.expanduser('~')+"/.scopy"
		if not os.path.exists(d):
			os.makedirs(d)
			config = open(os.path.expanduser('~')+"/.scopy/config", 'w')
			config.close()
			self.opzioni = {}
			for key in self.default:
				self.opzioni[key] = self.default[key]
			self.save()
		else:
			try:
				self.opzioni = self.parse(os.path.expanduser('~')+"/.scopy/config")
				for key in self.default:
					if not key in self.opzioni:
						self.opzioni[key] = self.default[key]
				if self.opzioni['variante'] not in varianti:
					self.opzioni['variante'] = self.default['variante']
				self.save()
			except:
				self.opzioni = {}
				for key in self.default:
					self.opzioni[key] = self.default[key]
				self.save()
	def save(self):
		config = open(os.path.expanduser('~')+"/.scopy/config", 'w')
		for key in self.opzioni:
			config.write(key+'='+self.opzioni[key]+'\n')
		config.close()
	def parse(self, file_name):
		lang = {}
		f = open(file_name)
		riga = f.readline()
		while riga != '':
			try:
				pos_uguale = riga.index('=')
			except:
				break
			lang[riga[0:pos_uguale]]=riga[pos_uguale+1:-1]
			riga = f.readline()
		return lang
	#crea una nuova partita in base ai dati in self.opzioni
	def crea_partita(self, nc=None):
		if nc==None:
			nc=_('Computer')
		if self.opzioni['variante'] == None:
			import classica
			self.partita = classica.partita(self.opzioni['nome'],nc)
		if self.opzioni['variante'] == _('Classic scopa'):
			import classica
			self.partita = classica.partita(self.opzioni['nome'],nc)
		if self.opzioni['variante'] == _('Cirulla'):
			import cirulla
			self.partita = cirulla.partita(self.opzioni['nome'],nc)
		if self.opzioni['variante'] == _('Cucita'):
			import cucita
			self.partita = cucita.partita(self.opzioni['nome'],nc)
		if self.opzioni['variante'] == _('Re Bello'):
			import re_bello
			self.partita = re_bello.partita(self.opzioni['nome'],nc)
	#dichiara le combinazioni di carte, es:Cirulla
	def dichiara(self, dichiarazione):
		for n in range(2):
			if dichiarazione != None:
				testo=''
				if 0 in dichiarazione[n]:
					testo = _('Sum lower then 9')
					if 1 in dichiarazione[n]:
						testo = testo + _(' and 3 egual cards')
				elif 1 in dichiarazione[n]:
						testo = testo + _('3 egual cards')
				self.dichiarazione[n].set_text(testo)
		GLib.timeout_add(5000, self.free_dichiarazioni, [0,1])
	#setta il testo delle dichiarazioni a ''
	def free_dichiarazioni(self, lista):
		for ide in lista:
			self.dichiarazione[ide].set_text('')
	#setta che il giocatore può giocare => se si clicka su una carta viene eseguita self.gioca_giocatore
	def set_giocatore_can_play(self, widget=None):
		self.giocatore_can_play = 1
	#funzione che mostra l'interfaccia per scegliere una presa
	def scelta_presa(self, obj, prese_possibili):
		self.dichiarazione[1].set_text(_('What do you take?'))
		i=self.stage.get_n_children()
		n=0
		while n<i:
			if self.stage.get_nth_child(n) != self.back_img:
				self.stage.get_nth_child(n).hide()
			n=n+1
		oggetti=[]
		n=0
		while n<len(prese_possibili):
			i=0
			while i < len(prese_possibili[n]):
				carta = self.partita.carte_terra[prese_possibili[n][i]]
				actor = Clutter.Texture.new_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
				actor.set_position(self.pos_sce[n][i][0],self.pos_sce[n][i][1])
				actor.set_reactive(True)
				self.stage.add_actor(actor)
				actor.connect('button-press-event',self.scelta_fatta,oggetti,obj,n)
				oggetti.append(actor)
				i=i+1
			n=n+1
	#funzione eseguita quando si è scelta una presa
	def scelta_fatta(self, actor,data,oggetti,obj,n):
		self.dichiarazione[1].set_text('')
		for Obj in oggetti:
			Obj.destroy()
		i=self.stage.get_n_children()
		g=0
		while g<i:
			self.stage.get_nth_child(g).show()
			g=g+1
		self.set_giocatore_can_play()
		self.gioca_giocatore(obj,data, n)
	#mostra l'ultima presa
	def vedi_ultima_presa(self,widget,data=None):
		if self.ultima_presa[0] != 0:
			widget.set_label(_('Hide last move'))
			self.dichiarazione[1].set_text(_('Computer played ... and took ...'))
			i=self.stage.get_n_children()
			n=0
			while n<i:
				if self.stage.get_nth_child(n) != self.back_img:
					self.stage.get_nth_child(n).hide()
				n=n+1
			oggetti=[]
			actor = Clutter.Texture.new_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[self.ultima_presa[0][0]][self.ultima_presa[0][1]])
			actor.set_position(10,10)
			actor.set_reactive(True)
			self.stage.add_actor(actor)
			actor.connect('button-press-event',self.nascondi_ultima_presa,widget,oggetti)
			oggetti.append(actor)
			i=0
			while i < len(self.ultima_presa[1]):
				actor = Clutter.Texture.new_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[self.ultima_presa[1][i][0]][self.ultima_presa[1][i][1]])
				actor.set_position(10+i*70,160)
				actor.set_reactive(True)
				self.stage.add_actor(actor)
				actor.connect('button-press-event',self.nascondi_ultima_presa,widget,oggetti)
				oggetti.append(actor)
				i=i+1
			widget.disconnect_by_func(self.vedi_ultima_presa)
			widget.connect('activate', self.nascondi_ultima_presa,None,widget,oggetti)
	#nasconde l'ultima presa
	def nascondi_ultima_presa(self, actor, event,widget, oggetti):
		widget.set_label(_('Show last move'))
		widget.disconnect_by_func(self.nascondi_ultima_presa)
		widget.connect('activate', self.vedi_ultima_presa)
		self.dichiarazione[1].set_text('')
		for obj in oggetti:
			obj.destroy()
		i=self.stage.get_n_children()
		n=0
		while n<i:
			self.stage.get_nth_child(n).show()
			n=n+1
	#restituisce la prima posizione libera a terra
	def primo_libero(self):
		n=0
		while n<10:
			if self.posizioni['terra_c'][n] == 0:
				return n
			n=n+1
		return n-1
	def presa_da_terra(self, carta, prese, giocatore):
		an = go_to(carta, self.posizioni['prese'][giocatore][0],self.posizioni['prese'][giocatore][1], self.time_go)
		if self.partita.giocatore[giocatore].scope != 0:
			an.connect('completed', self.caller, self.n_scope[giocatore].set_markup, "<span background='#000' color='#FFF'> "+str(self.partita.giocatore[giocatore].scope)+' </span>')
			an.connect('completed', self.caller, self.n_scope[giocatore].show)
		if giocatore == 1:
			an.connect('completed', self.caller, self.set_giocatore_can_play)
		an.connect('completed', self.caller, self.scope[giocatore].set_from_file, percorso_carte+self.opzioni['carte']+'/'+immagini[self.partita.giocatore[giocatore].ult_scopa[0]][self.partita.giocatore[giocatore].ult_scopa[1]])
		an.connect('completed', self.caller, carta.destroy)
		for Obj in prese:
			an=go_to(Obj, self.posizioni['prese'][giocatore][0]-self.posizioni['prese'][giocatore][2],
				self.posizioni['prese'][giocatore][1]+self.posizioni['prese'][giocatore][2], self.time_go)
			self.posizioni['terra_c'][self.posizioni['terra_c'].index(Obj)]=0
			an.connect('completed', self.caller, Obj.set_from_file, percorso_carte+self.opzioni['carte']+'/'+immagini[1][0])
			self.posizioni['prese'][giocatore][2]=self.posizioni['prese'][giocatore][2]+1
	def caller(actor, event, callback, *data):
		callback(*data)
		
	#gioca computer
	def gioca_computer(self,giocata=None):
		if len(self.partita.get_carte_giocatore(0)) == 0 and len(self.partita.get_carte_giocatore(1)) == 0:
			if len(self.partita.mazzo.carte) == 0:
				self.show_riepilogo()
			else:
				for n in range(2):
					if self.partita.giocatore[n].scope != 0:
						GLib.timeout_add(self.time_go, self.n_scope[n].set_markup, "<span background='#000' color='#FFF'> "+str(self.partita.giocatore[n].scope)+' </span>')
						GLib.timeout_add(self.time_go, self.n_scope[n].show)
				self.dichiara(self.partita.dai_carte())
				self.sposta_carte()
		else:
			if giocata == None:
				giocata = self.partita.gioca_computer()
			carta = self.partita.giocatore[1].mano[giocata[0]]
			Obj = self.carte[giocata[0]]
			Obj.set_from_file(percorso_carte+self.opzioni['carte']+'/'+immagini[carta.palo][carta.valore])
			self.ultima_presa[0]=[carta.palo,carta.valore]
			self.ultima_presa[1] = []
			for uid in giocata[1]:
				card = self.partita.carte_terra[uid]
				self.ultima_presa[1].append([card.palo,card.valore])
			self.partita.gioca_carta(1, giocata[0], giocata[1])
			if len(giocata[1]) == 0:
				n=self.primo_libero()
				go_to(Obj,self.posizioni['terra'][n][0], self.posizioni['terra'][n][1], self.time_go)
				self.posizioni['terra_c'][n] = Obj
				GLib.timeout_add(self.time_go, self.set_giocatore_can_play)
			else:
				n=self.primo_libero()
				go_to(Obj,self.carte[giocata[1][0]].get_x()+30, self.carte[giocata[1][0]].get_y()+30, self.time_go)
				del self.carte[self.carte[Obj]]
				prese=[]
				for carta in giocata[1]:
					prese.append(self.carte[carta])
				GLib.timeout_add(self.time_go+self.time, self.presa_da_terra,Obj,prese,1)
			if len(self.partita.get_carte_giocatore(0)) == 0 and len(self.partita.get_carte_giocatore(1)) == 0:
				if len(self.partita.mazzo.carte) == 0:
					GLib.timeout_add(2*self.time_go+self.time+500, self.show_riepilogo)
				else:
					dic=self.partita.dai_carte()
					GLib.timeout_add(2*self.time_go+self.time+500,self.dichiara, dic)
					GLib.timeout_add(self.time_go+self.time+500, self.sposta_carte)
					for n in range(2):
						if self.partita.giocatore[n].scope != 0:
							GLib.timeout_add(2*self.time_go+self.time+500, self.n_scope[n].set_markup, "<span background='#000' color='#FFF'> "+str(self.partita.giocatore[n].scope)+' </span>')
							GLib.timeout_add(2*self.time_go+self.time+500, self.n_scope[n].show)
	
	#Questa funzione e' chiamata quando il giocatore clicka su una carta
	def gioca_giocatore(self, Obj, event, data=None):
		if self.giocatore_can_play == 1:	
			self.giocatore_can_play = 0
			ide_carta = self.carte[Obj]
			prese_possibili = self.partita.gioca_giocatore(ide_carta)
			if self.online_mode == True:
				if len(prese_possibili) < 2 or data != None:
					if len(prese_possibili) < 2:
						data = 0
					self.client.send_message(mto=self.playing_user,
						mbody='scopy:partita'+pickle.dumps([self.partita,[ide_carta,prese_possibili[data]]]),
						mtype='chat')
			if len(prese_possibili[0]) == 0:
				self.partita.gioca_carta(0, ide_carta, prese_possibili[0])
				n=self.primo_libero()
				go_to(Obj,self.posizioni['terra'][n][0], self.posizioni['terra'][n][1], self.time_go)
				self.posizioni['terra_c'][n] = Obj
				if self.online_mode == False:
					GLib.timeout_add(1000, self.gioca_computer)
			else:
				if len(prese_possibili) == 1:
					data=0
				if data == None:
					self.scelta_presa(Obj,prese_possibili)
				else:
					go_to(Obj,self.carte[prese_possibili[data][0]].get_x()+30, self.carte[prese_possibili[data][0]].get_y()+30, self.time_go)
					self.partita.gioca_carta(0, ide_carta, prese_possibili[data])
					del self.carte[self.carte[Obj]]
					prese=[]
					for carta in prese_possibili[data]:
						prese.append(self.carte[carta])
					GLib.timeout_add(self.time_go+self.time, self.presa_da_terra,Obj,prese,0)
					if self.online_mode == False:
						GLib.timeout_add(self.time_go+self.time+500, self.gioca_computer)
			if self.online_mode == True and len(self.partita.get_carte_giocatore(0)) == 0 and len(self.partita.get_carte_giocatore(1)) == 0:
				if len(self.partita.mazzo.carte) == 0:
					GLib.timeout_add(2*self.time_go+self.time+500, self.show_riepilogo)
	#sposta le carte a terra e ai giocatori a inizio mano
	def sposta_carte(self):
		for n in range(2):
			if self.partita.giocatore[n].scope != 0:
				GLib.timeout_add(self.time_go, self.n_scope[n].set_markup, "<span background='#000' color='#FFF'> "+str(self.partita.giocatore[n].scope)+' </span>')
				GLib.timeout_add(self.time_go, self.n_scope[n].show)
		c_id = self.status_bar.get_context_id('situazione')
		self.status_bar.push(c_id,
			_('Remaining rounds:')
			+' '+str(len(self.partita.mazzo.carte)/6)
			+'  '+self.partita.giocatore[1].nome+': '+str(self.partita.giocatore[1].punti)
			+'  '+self.partita.giocatore[0].nome+': '
			+str(self.partita.giocatore[0].punti)
			)
		time =100
		#a terra
		if self.partita.mano == 1:
			n=0
			while n<4:
				carta = self.partita.carte_terra.carte[n]
				obj = Obj(self.stage, carta, self.opzioni, True)
				self.carte.append(obj)
				go_to(self.carte[carta.uid],self.posizioni['terra'][n][0],self.posizioni['terra'][n][1],self.time_go)
				self.posizioni['terra_c'][n] = self.carte[carta.uid]
				n=n+1
		#al computer
		n=0
		while n<3:
			carta = self.partita.get_carte_giocatore(1)[n]
			if self.partita.giocatore[1].scoperte == 0:
				obj = Obj(self.stage, carta, self.opzioni, False)
			else:
				obj = Obj(self.stage, carta, self.opzioni, True)
			self.carte.append(obj)
			go_to(self.carte[carta.uid],self.posizioni['computer'][n][0],self.posizioni['computer'][n][1],self.time_go)
			n=n+1
		#al giocatore
		n=0
		while n<3:
			carta = self.partita.get_carte_giocatore(0)[n]
			obj = Obj(self.stage, carta, self.opzioni, True)
			self.carte.append(obj)
			go_to(self.carte[carta.uid],self.posizioni['giocatore'][n][0],self.posizioni['giocatore'][n][1],self.time_go)
			self.carte[carta.uid].set_reactive(True)
			self.carte[carta.uid].connect('button-press-event',self.gioca_giocatore)
			n=n+1
		if self.partita.ide == 0:
			self.giocatore_can_play=1
			if self.partita.mano == 1:
				GLib.timeout_add(self.time_go,self.comunica,_("It's your turn"), 1000)
		else:
			if self.partita.mano == 1:
				GLib.timeout_add(self.time_go,self.comunica,_('I start'), 1000)
			if self.online_mode == False:
				GLib.timeout_add(2000, self.gioca_computer)
	#serve perchè i Clutter.FuckedActor non ricevono i segnali a dovere
	def button_press(self, actor,data):
		actor = self.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE,data.x,data.y)
		if actor != self.stage:
			event=Clutter.Event.new(Clutter.EventType.BUTTON_PRESS)
			actor.event(event,False)
	#scrive 'messaggio' al centro dello schermo per 'time' ms
	def comunica(self, messaggio, time):
		text=Clutter.Text()
		pango = "<span background='#FFF'>   "+messaggio+'   </span>'
		text.set_markup(pango)
		text.set_position(self.width/3,self.height/2)
		self.stage.add_actor(text)
		GLib.timeout_add(time, text.destroy)
main_win = main_win()
Gtk.main()
